/**
 * Copyright 2017 ACCELECOM LTD.  All Rights Reserved.
 *  @file userdemo2.cpp
 *  @company 南京艾科朗克信息科技有限公司
 *  @author xinweizhang
 *  @modify
 *  @source encoding UTF-8
 *  @console encoding GB2312
 *  @Formatter GNU Built-in
 *  @brief 一个简单的例子，介绍CXeleTraderApi和CXeleTraderSpi接口的使用。
 *  本例将演示4个业务的基本流程,以及以下查询的结构体构建：
 *    1. 持仓查询(Position)
 *    2. 报单查询(Order)
 *    3. 成交单查询(Trade)
 *    4. 资金查询(Account)
 */

#include <iostream>
#include <iomanip>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <assert.h>
#include <CXeleTraderApi.hpp>
#include <sched.h>
#include <unistd.h>

////////////////////////////////////////////////////////////////
/// BEGIN 编辑以下信息，用以测试柜台功能

// 柜台连接信息
static const char * FRONT_ADDRESS = "udp://10.128.123.84:32001";
static const char * QUERY_FRONT_ADDRESS = "tcp://192.168.2.8:33333";
static const char * LOCAL_ADDRESS = "udp://192.168.2.8:32001";

// 账号信息
static const char * ACCOUNT_ID = "00000000";
static const char * PASSWD = "000000";
static const char * AUTHCODE = "00000000";
static const char * APPID = "xele_trade-0.0";

// 双交易所的报单信息
static const char * INSTRUMENT_ID[2]{"cu2305", "si2303"};
static int          VOLUME[2]{5, 3};
static int          MIN_VOLUME[2]{5, 3};
static double       PRICE[2]{14382, 123};
static int          INSERT_TYPE[2]{0x01, 0x02};
static int          EXCHANGE_FRONT[2]{0, 0};

// 查询指定的成交信息
static const char * TRADE_ID = "233";

/// END 编辑以下信息，用以测试柜台功能
////////////////////////////////////////////////////////////////


// 批量报单最大数量
static int          MAX_ORDER_NUM = 16;

#define LOG(X, ...) fprintf(stdout, "[%s:line %04d]: " X "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__)

// 报单帮助类
class OrderHelper {
public:
    static void fillReqUserLogin(CXeleFtdcReqUserLoginField &req) {
        memset(&req, 0, sizeof(req));
        snprintf(req.AccountID, sizeof(req.AccountID), "%s", ACCOUNT_ID);
        snprintf(req.Password, sizeof(req.Password), "%s", PASSWD);
    }

    static void fillRegisterAuthentication(CXeleFtdcAuthenticationInfoField &authinfo) {
        memset(&authinfo, 0, sizeof(authinfo));
        snprintf(authinfo.AuthCode, sizeof(authinfo.AuthCode), "%s", AUTHCODE);
        snprintf(authinfo.AppID, sizeof(authinfo.AppID), "%s", APPID);
    }

    // 构造报单
    static void makeupOrder(CXeleFairInputOrderField *order,
                            uint8_t clientIndex,
                            uint16_t token,
                            const char *instrumentID,
                            double price,
                            int volume,
                            int minVolume,
                            int insertType,
                            int exchangeFront,
                            int orderLocalNo) {
        order->OrderLocalNo = orderLocalNo;
        order->LimitPrice = price;
        order->ClientIndex = clientIndex;
        order->ClientToken = token;
        strcpy(order->InstrumentID, instrumentID); // 合约代码
        order->VolumeTotalOriginal = volume;
        order->MinVolume = minVolume;
        order->InsertType = insertType;
        order->ExchangeFront = exchangeFront;
        LOG("order: InstrumentID=%s, LimitPrice=%.4f, VolumeTotalOriginal=%d, MinVolume=%d, "
            "InsertType=%d, ExchangeFront=%d, OrderLocalNo=%d, ClientIndex=%d, ClientToken=0x%x",
            instrumentID, price, volume, minVolume, insertType, exchangeFront, orderLocalNo, clientIndex, token);
    }

    // 构造撤单
    static void makeupCancelOrder(CXeleFairOrderActionField *action,
                                  uint8_t clientIndex,
                                  uint16_t token,
                                  int orderSysNo) {
        action->OrderSysNo = orderSysNo;
        action->ClientIndex = clientIndex;
        action->ClientToken = token;
        action->ActionFlag = '0';
        LOG("cancel order: OrderSysNo=%d, ClientIndex=%d, ClientToken=0x%x", orderSysNo, clientIndex, token);
    }

    // 批量委托单，构造报单
    static void makeupOrderBatch(CXeleFairInputOrderField *order,
                                 uint8_t clientIndex,
                                 uint16_t token,
                                 int requestId,
                                 const char *instrumentID,
                                 double price,
                                 int volume,
                                 int minVolume,
                                 int insertType,
                                 int exchangeFront,
                                 int orderLocalNo) {
        order->ProductReserve1 = 0x65;
        order->ProductReserve3 = requestId;
        order->OrderLocalNo = orderLocalNo;
        order->LimitPrice = price;
        order->ClientIndex = clientIndex;
        order->ClientToken = token;
        strcpy(order->InstrumentID, instrumentID);
        order->VolumeTotalOriginal = volume;
        order->MinVolume = minVolume;
        order->InsertType = insertType;
        order->ExchangeFront = exchangeFront;
        LOG("batch order: InstrumentID=%s, LimitPrice=%.4f, VolumeTotalOriginal=%d, MinVolume=%d, "
            "InsertType=%d, ExchangeFront=%d, RequestID=%d, OrderLocalNo=%d, ClientIndex=%d, ClientToken=0x%x",
            instrumentID, price, volume, minVolume, insertType, exchangeFront, requestId, orderLocalNo, clientIndex, token);
    }

    // 批量委托单，构造撤单
    static void makeupCancelOrderbatch(CXeleFairOrderActionField *action,
                                       uint8_t clientIndex,
                                       uint16_t token,
                                       int orderSysNo,
                                       int requestId) {
        action->ProductReserve1 = 0x67;
        action->ProductReserve3 = requestId;
        action->ClientIndex = clientIndex;
        action->ClientToken = token;
        action->OrderSysNo = orderSysNo;
        action->ActionFlag = '0';
        LOG("batch cancel order: RequestID=%d, OrderSysNo=%d, ClientIndex=%d, ClientToken=0x%x",
            requestId, orderSysNo, clientIndex, token);
    }

    // 响应信息
    static void printResponse(CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast) {
        printf("[PASSED] ErrorID: %d, ErrorMsg: %s nRequestID: %d Chain: %d.\n", pRspInfo->ErrorID, pRspInfo->ErrorMsg, nRequestID, bIsLast);
    }

    static void printResponseError(CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast) {
        printf("[FAILED] ErrorID: %d, ErrorMsg: %s nRequestID: %d Chain: %d.\n", pRspInfo->ErrorID, pRspInfo->ErrorMsg, nRequestID, bIsLast);
    }
};


/*
 * @class UserTrader
 * @brief 利用报单管理场景，演示本API接口调用。
 * @notes CXeleTraderSpi中OnXXX()形式的方法为回调函数，不能阻塞过长时间,
 *        否则，会影响接收其他来自交易柜台的消息。
 */
class UserTrader : public CXeleTraderSpi {
public:
    /* 构造函数，需要一个有效的指向CXeleTraderApi实例的指针 */
    UserTrader(CXeleTraderApi *pTraderApi) : mTraderApi(pTraderApi) {}
    ~UserTrader() = default;

    /* 当客户端程序与交易柜台建立起通信连接，客户端程序需要进行登录 */
    void OnFrontConnected() override {
        LOG("front connected.");
        OrderHelper::fillReqUserLogin(mLoginInfo);
        OrderHelper::fillRegisterAuthentication(mAuthInfo);
        mTraderApi->RegisterAuthentication(&mAuthInfo);

        LOG("start login...");
        mRspEvent = false;
        mTraderApi->ReqUserLogin(&mLoginInfo, 0);
    }

    /* 当客户端程序与交易柜台通信连接断开时，该方法被调用 */
    void OnFrontDisconnected(int nReason) override {
        LOG("front disconnected, reason: %d", nReason);
    }

    /* 当客户端程序发出登录请求之后，该方法会被调用，通知客户端程序登录是否成功  */
    void OnRspUserLogin(CXeleFtdcRspUserLoginField *pRspUserLogin,
                        CXeleFtdcRspInfoField *pRspInfo,
                        int nRequestID,
                        bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            LOG("login failed, error code: %d", pRspInfo->ErrorID);
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            exit(pRspInfo->ErrorID);
        } else {
            LOG("login success");
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            mExchangeNum = pRspUserLogin->ExchangeNum;
            mClientIndex[0] = pRspUserLogin->ClientIndex[0];
            mToken[0] = pRspUserLogin->Token[0];
            mClientIndex[1] = pRspUserLogin->ClientIndex[1];
            mToken[1] = pRspUserLogin->Token[1];
            LOG("OnRspUserLogin: ExchangeNum=%d, ClientIndex0=%d, Token0=0x%x, ClientIndex1=%d, Token1=0x%x",
                mExchangeNum, mClientIndex[0], mToken[0], mClientIndex[1], mToken[1]);
        }

        mRspEvent = true;
    }

    void OnRspOrderInsert(CXeleFtdcInputOrderField *pInputOrder,
                          CXeleFtdcRspInfoField *pRspInfo,
                          int nRequestID,
                          bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            if (mOrderCount < mExchangeNum) {
                mOrderSysNo[mOrderCount++] = pInputOrder->OrderSystemNo;
            }
        }

        mRspEvent = true;
    }


    void OnRspOrderAction(CXeleFtdcOrderActionField *pOrderAction,
                          CXeleFtdcRspInfoField *pRspInfo,
                          int nRequestID,
                          bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
        }

        mRspEvent = true;
    }

    /* 报单回报 */
    void OnRtnOrder(CXeleFtdcOrderField *pOrder) override {
        LOG("OrderLocalNo=%d, OrderSystemNo=%d", pOrder->OrderLocalNo, pOrder->OrderSystemNo);
    }

    /* 针对用户请求的出错通知 */
    void OnRspError(CXeleFtdcRspInfoField *pRspInfo, int nRequestID, bool bIsLast) override {
        OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
        //exit(pRspInfo->ErrorID);
    }

    /* 持仓查询请求 */
    void ReqQryClientPosition() {
        CXeleFtdcQryClientPositionField qryPosition;
        memset(&qryPosition, 0, sizeof(CXeleFtdcQryClientPositionField));
        snprintf(qryPosition.AccountID, sizeof(qryPosition.AccountID), "%s", ACCOUNT_ID); // 客户代码

        LOG("query client position: %s", ACCOUNT_ID);
        mRspEvent = false;
        mTraderApi->ReqQryClientPosition(&qryPosition, 1);
    }

    /* 持仓查询应答 */
    void OnRspQryClientPosition(CXeleFtdcRspClientPositionField *pRspClientPosition,
                                CXeleFtdcRspInfoField *pRspInfo,
                                int nRequestID,
                                bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast);
            std::cout << "position";
            std::cout << ": instrument=" << pRspClientPosition->InstrumentID;
            std::cout << ", LongYdPosition=" << pRspClientPosition->LongYdPosition;
            std::cout << ", LongPosition=" << pRspClientPosition->LongPosition;
            std::cout << ", ShortYdPosition=" << pRspClientPosition->ShortYdPosition;
            std::cout << ", ShortPosition=" << pRspClientPosition->ShortPosition;
            std::cout << ", PositionCost=" << pRspClientPosition->PositionCost << std::endl;
        }

        mRspEvent = true;
    }

    /* 资金查询请求 */
    void ReqQryClientAccount() {
        CXeleFtdcQryClientAccountField qryAccount;
        memset(&qryAccount, 0, sizeof(CXeleFtdcQryClientAccountField));
        snprintf(qryAccount.AccountID, sizeof(qryAccount.AccountID), "%s", ACCOUNT_ID); // 资金帐号

        LOG("query client: %s", ACCOUNT_ID);
        mRspEvent = false;
        mTraderApi->ReqQryClientAccount(&qryAccount, 1);
    }

    /* 资金查询应答 */
    void OnRspQryClientAccount(CXeleFtdcRspClientAccountField *pClientAccount,
                               CXeleFtdcRspInfoField *pRspInfo,
                               int nRequestID,
                               bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            std::cout << "Account: " << std::endl;
            std::cout << "  TradingDay     : " << pClientAccount->TradingDay     << std::endl;
            std::cout << "  AccountID      : " << pClientAccount->AccountID      << std::endl;
            std::cout << "  PreBalance     : " << pClientAccount->PreBalance     << std::endl;
            std::cout << "  CurrMargin     : " << pClientAccount->CurrMargin     << std::endl;
            std::cout << "  CloseProfit    : " << pClientAccount->CloseProfit    << std::endl;
            std::cout << "  Premium        : " << pClientAccount->Premium        << std::endl;
            std::cout << "  Deposit        : " << pClientAccount->Deposit        << std::endl;
            std::cout << "  Withdraw       : " << pClientAccount->Withdraw       << std::endl;
            std::cout << "  Balance        : " << pClientAccount->Balance        << std::endl;
            std::cout << "  Available      : " << pClientAccount->Available      << std::endl;
            std::cout << "  FrozenMargin   : " << pClientAccount->FrozenMargin   << std::endl;
            std::cout << "  FrozenPremium  : " << pClientAccount->FrozenPremium  << std::endl;
            std::cout << "  floatProfitAndLoss: " << pClientAccount->floatProfitAndLoss << std::endl;
        }

        mRspEvent = true;
    }

    /* 报单查询请求 */
    void ReqQryOrder() {
        CXeleFtdcQryOrderField qryOrder;
        memset(&qryOrder, 0, sizeof(CXeleFtdcQryOrderField));
        snprintf(qryOrder.AccountID, sizeof(qryOrder.AccountID), "%s", ACCOUNT_ID); // 客户代码

        LOG("query orders: %s", ACCOUNT_ID);
        mRspEvent = false;
        mTraderApi->ReqQryOrder(&qryOrder, 1);
    }

    /* 报单查询应答 */
    void OnRspQryOrder(CXeleFtdcOrderField *pOrderField,
                       CXeleFtdcRspInfoField *pRspInfo,
                       int nRequestID,
                       bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            std::cout << "order: " << pOrderField->ExchangeOrderSysID
                      << ", status: " << pOrderField->OrderStatus
                      << ", instrument id: " << pOrderField->InstrumentID
                      << ", system id: " << pOrderField->OrderSystemNo
                      << std::endl;
        }

        mRspEvent = true;
    }

    /* 成交单查询请求 */
    void ReqQryTrade() {
        CXeleFtdcQryTradeField qryTrade;
        memset(&qryTrade, 0, sizeof(CXeleFtdcQryTradeField));
        snprintf(qryTrade.AccountID, sizeof(qryTrade.AccountID), "%s", ACCOUNT_ID); // 客户代码
        //strcpy(qryTrade.InstrumentID, INSTRUMENT_ID[0]); // 合约代码
        //strcpy(qryTrade.TradeID, TRADE_ID); // 成交编号

        LOG("query trade, account: %s, trade id: %s(unused)", ACCOUNT_ID, TRADE_ID);
        mRspEvent = false;
        mTraderApi->ReqQryTrade(&qryTrade, 1);
    }

    /* 成交单查询应答 */
    void OnRspQryTrade(CXeleFtdcTradeField *pTradeField,
                       CXeleFtdcRspInfoField *pRspInfo,
                       int nRequestID,
                       bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            std::cout << "trade id: " << pTradeField->TradeID
                      << ", order id: " << pTradeField->ExchangeOrderSysID
                      << ", trade type: " << pTradeField->TradeType
                      << ", instrument id: " << pTradeField->InstrumentID
                      << ", system id: " << pTradeField->OrderSystemNo
                      << std::endl;
        }

        mRspEvent = true;
    }

    /* 交易所前置查询请求 */
    void ReqQryExchangeFront() {
        LOG("query exchange front");
        mRspEvent = false;
        mTraderApi->ReqQryExchangeFront(1);
    }

    /* 交易所前置查询应答 */
    void OnRspQryExchangeFront(CXeleFtdcRspExchangeFrontField *pRspExchangeFront,
                               CXeleFtdcRspInfoField *pRspInfo,
                               int nRequestID,
                               bool bIsLast) override {
        if (pRspInfo->ErrorID != 0) {
            OrderHelper::printResponseError(pRspInfo, nRequestID, bIsLast); // 失败，客户端程序需进行错误处理
            //exit(pRspInfo->ErrorID);
        } else {
            OrderHelper::printResponse(pRspInfo, nRequestID, bIsLast); // 成功，处理数据...
            printf("ExchangeID: %d, isLast=%s\n", pRspExchangeFront->ExchangeID, (bIsLast ? "true" : "false"));
            for (int i = 0; i < pRspExchangeFront->FrontCount; ++i) {
                int seatId = static_cast<int>(pRspExchangeFront->FrontList[i]);
                std::string frontIp = pRspExchangeFront->FrontIpList[seatId - 10];
                printf("  Front-%d: SeatId=%d, FrontIp=%s\n", i, seatId, frontIp.c_str());
            }
        }
        mRspEvent = true;
    }

    /* 合约交易状态通知回报 */
    void OnRtnInstrumentStatus(CXeleFtdcInstrumentStatusField *pInstrumentStatus) override {
        LOG("InstrumentID=[%s],InstrumentStatus=[%c]",
            pInstrumentStatus->InstrumentID, pInstrumentStatus->InstrumentStatus);
    }

    /* 等待响应事件 */
    void WaitRspEvent() {
        do {
            sched_yield();
        } while (!mRspEvent);
    }

    bool runQueryTest() {
        ReqQryExchangeFront(); // 交易所前置查询
        sleep(2);
        WaitRspEvent();

        ReqQryClientAccount(); // 资金查询应答
        sleep(2);
        WaitRspEvent();

        ReqQryClientPosition(); // 持仓查询应答
        sleep(2);
        WaitRspEvent();

        ReqQryTrade(); // 成交单查询请求
        sleep(2);
        WaitRspEvent();

        ReqQryOrder(); // 报单查询请求
        sleep(2);
        WaitRspEvent();

        return true;
    }

    void runOrderTest() {
        int i = 0;
        CXeleFairInputOrderField order;
        CXeleFairOrderActionField orderAction;
        CXeleFairOrderFieldV orderActions[MAX_ORDER_NUM];

        WaitRspEvent(); // 等待登入成功
        runQueryTest(); // 查询接口

        // 报单接口
        mOrderCount = 0;
        for (i = 0; i < mExchangeNum; i++) {
            memset(&order, 0, sizeof(CXeleFairInputOrderField));
            OrderHelper::makeupOrder(&order, mClientIndex[i], mToken[i],
                                     INSTRUMENT_ID[i], PRICE[i], VOLUME[i], MIN_VOLUME[i],
                                     INSERT_TYPE[i], EXCHANGE_FRONT[i], ++mOrderLocalNo);
            mTraderApi->ReqOrderInsert(&order, ++mRequestID);
            sleep(2);
            WaitRspEvent();
        }

        // 撤单接口
        for (i = 0; i < mExchangeNum; i++) {
            memset(&orderAction, 0, sizeof(CXeleFairOrderActionField));
            OrderHelper::makeupCancelOrder(&orderAction, mClientIndex[i], mToken[i], mOrderSysNo[i]);
            mTraderApi->ReqOrderAction(&orderAction, ++mRequestID);
            sleep(2);
            WaitRspEvent();
        }

        // 批量委托报单接口
        memset(orderActions, 0, sizeof(orderActions));
        for (i = 0; i < mExchangeNum; i++) {
            OrderHelper::makeupOrderBatch(&orderActions[i].order.input,
                                          mClientIndex[i], mToken[i], ++mRequestID,
                                          INSTRUMENT_ID[i], PRICE[i], VOLUME[i], MIN_VOLUME[i],
                                          INSERT_TYPE[i], EXCHANGE_FRONT[i], ++mOrderLocalNo);
        }
        mTraderApi->ReqOrderInsertActionV(orderActions, mExchangeNum);
        sleep(2);
        WaitRspEvent();

        // 批量委托撤单接口
        memset(orderActions, 0, sizeof(orderActions));
        for (i = 0; i < mExchangeNum; i++) {
            OrderHelper::makeupCancelOrderbatch(&orderActions[i].order.action,
                                                mClientIndex[i],
                                                mToken[i],
                                                mOrderSysNo[i],
                                                ++mRequestID);
        }
        mTraderApi->ReqOrderInsertActionV(orderActions, mExchangeNum);
        sleep(2);
        WaitRspEvent();
    }

private:
    /* 指向CXeleTraderApi实例的指针 */
    CXeleTraderApi *mTraderApi;

    /* 响应事件标志 */
    bool        mRspEvent;
    uint32_t    mOrderCount;
    int         mRequestID = 0;
    int         mOrderLocalNo = 0;
    int         mOrderSysNo[XELE_EXCHANGE_LOGIN_NUM];
    uint8_t     mExchangeNum;
    uint8_t     mClientIndex[XELE_EXCHANGE_LOGIN_NUM];
    uint16_t    mToken[XELE_EXCHANGE_LOGIN_NUM];
    CXeleFtdcReqUserLoginField       mLoginInfo;
    CXeleFtdcAuthenticationInfoField mAuthInfo;
};

void printConfigs() {
    printf("Front Address: %s\n", FRONT_ADDRESS);
    printf("Query Front Address: %s\n", QUERY_FRONT_ADDRESS);
    printf("Local Address: %s\n", LOCAL_ADDRESS);
    printf("Account ID: %s\n", ACCOUNT_ID);
    printf("Password: %s\n", PASSWD);
    printf("Auth Code: %s\n", AUTHCODE);
    printf("App ID: %s\n", APPID);

    for (int i = 0; i < 2; ++i) {
        printf("\n");
        printf("Exchange-%d Order Configs:\n", i);
        printf("  Instrument ID: %s\n", INSTRUMENT_ID[i]);
        printf("  Volume: %d\n", VOLUME[i]);
        printf("  Min Volume: %d\n", MIN_VOLUME[i]);
        printf("  Price: %.4f\n", PRICE[i]);
        printf("  Insert Type: %d\n", INSERT_TYPE[i]);
        printf("  Exchange Front: %d\n", EXCHANGE_FRONT[i]);
    }
}

int main(int argc, char *argv[]) {
    std::cout << std::fixed
              << std::setprecision(4)
              << "xtf-api-client, build-time: "
              << __DATE__ << " " << __TIME__ << std::endl;

    CXeleTraderApi *api = CXeleTraderApi::CreateTraderApi(0);
    assert(api);

    printf("%s\n\n", api->GetVersion()); // 打印API信息
    printConfigs();

    UserTrader userTrader(api);
    api->RegisterSpi(&userTrader);

    char *frontAddress = strdup(FRONT_ADDRESS);
    char *queryFrontAddress = strdup(QUERY_FRONT_ADDRESS);
    char *localAddress = strdup(LOCAL_ADDRESS);
    api->RegisterFront(frontAddress, queryFrontAddress, localAddress);
    free(frontAddress);
    free(queryFrontAddress);
    free(localAddress);

    // 默认报单通讯是非阻塞模式，若设置阻塞模式请在调用Init函数之前设置
    api->RegisterChannelBlock(1); // 设置为阻塞模式

    // 订阅相应的流
    api->SubscribePrivateTopic(XELE_TERT_RESTART);
    api->SubscribePublicTopic(XELE_TERT_RESTART);

    // 开始登录, 使客户端程序开始与交易柜台建立连接
    api->Init();

    // 实例流程：查询管理
    userTrader.runOrderTest();

    // 主线程打印, 等待退出
    std::string cmd;
    do {
        std::cout << "Input 'q' to disconnect API:";
        std::getline(std::cin, cmd);
    } while (cmd != "q");

    // 释放API实例
    api->Release();
    printf("API release done. Exit Demo.\n");

    return 0;
}
